<?php

error_reporting(E_ALL);
ini_set('display_errors', 0); // Set to 0 in production

// Log file path
$logFilePath = 'invitee_decline_logs.log';

// Function to log messages to a file
function logMessage($message) {
    global $logFilePath;
    $timestamp = date('Y-m-d H:i:s');
    file_put_contents($logFilePath, "[$timestamp] $message" . PHP_EOL, FILE_APPEND);
}

// Expected authentication token
$expectedToken = 'Motviz@754';

try {
    // Check if it's a validation request
    if ($_SERVER['REQUEST_METHOD'] === 'GET') {
        // Retrieve token from request headers
        $receivedToken = isset($_SERVER['HTTP_AUTHORIZATION']) ? $_SERVER['HTTP_AUTHORIZATION'] : '';

        // Validate the token
        if ($receivedToken === "Bearer $expectedToken" || $receivedToken === $expectedToken) {
            logMessage("Validation request successful.");
            http_response_code(200);
            echo json_encode(['message' => 'Validation successful']);
            exit;
        } else {
            logMessage("Validation request failed: Unauthorized.");
            http_response_code(401);
            echo json_encode(['error' => 'Unauthorized']);
            exit;
        }
    }

    // Handling POST requests (actual webhook messages)
    if ($_SERVER['REQUEST_METHOD'] === 'POST') {
        $data = file_get_contents('php://input');

        logMessage("Incoming data: $data");

        $filename = 'invitee_declined_responses.txt';
        file_put_contents($filename, "##### $data #####" . PHP_EOL, FILE_APPEND);

        $cvent_data = json_decode($data, true);

        // This helper function consists of all the necessary widely used functions
        include '../helper.php';
        include 'event_helper.php';

        $e_type = $cvent_data['eventType'];
        $e_code = $cvent_data['message'][0]['eventForInvitee']['eventCode'];
        $invitee_mail = $cvent_data['message'][0]['email'];
        $invitee_status = $cvent_data['message'][0]['inviteeStatus'];
        $token = access_token();

        $event_response_crm = event_search($e_code, $token);
        $e_id_crm = $event_response_crm->data[0]->id;

        // Use async or non-blocking approach here
        $contact_id = get_contact_id_async($token, $invitee_mail);

        if (($invitee_status == 'Declined' || $invitee_status == 'Cancelled') && !empty($e_id_crm) && !empty($contact_id) && $e_type == 'InviteeDeclined') {
            $event_data = array(
                'data' => [
                    array(
                        'id' => $e_id_crm,
                        'Member_Status' => 'Bounced'
                    )
                ]
            );

            $curl = curl_init();

            curl_setopt_array($curl, array(
                CURLOPT_URL => "https://www.zohoapis.in/crm/v6/Contacts/$contact_id/Campaigns",
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'PUT',
                CURLOPT_POSTFIELDS => json_encode($event_data),
                CURLOPT_HTTPHEADER => array(
                    "Authorization: Zoho-oauthtoken $token",
                    "Content-Type: application/json"
                ),
            ));

            $response = curl_exec($curl);
            curl_close($curl);

            echo $response;
        } else {
            echo json_encode(['message' => 'something wrong']);
        }
    }
} catch (Exception $e) {
    // Log any exceptions
    logMessage("Exception: " . $e->getMessage());
}

function get_contact_id_async($token, $invitee_mail) {
    $maxRetries = 10;
    $retryCount = 0;

    while ($retryCount < $maxRetries) {
        sleep(2); // Wait for 2 seconds before retrying
        $c_data = search_contact($token, $invitee_mail);

        if (!empty($c_data->data[0]->id)) {
            return $c_data->data[0]->id;
        }

        $retryCount++;
    }

    return null;
}
?>
