<?php

error_reporting(E_ALL);
ini_set('display_errors', 0); // Set to 0 in production

// Log file path
$logFilePath = 'invitee_mail_sent_logs.log';

// Function to log messages to a file
function logMessage($message) {
    global $logFilePath;
    $timestamp = date('Y-m-d H:i:s');
    file_put_contents($logFilePath, "[$timestamp] $message" . PHP_EOL, FILE_APPEND);
}

try {
    $data = file_get_contents('php://input');

    logMessage("Incoming data: $data");

    $filename = 'invitee_mail_sent_responses.txt';

    file_put_contents($filename, "##### $data #####", FILE_APPEND);

    $cvent_data = json_decode($data, true);

    // this helper function is consist of all the necessary widely used functions
    include '../helper.php';

    include 'event_helper.php';

    $e_type = $cvent_data['eventType'];
    $e_code = $cvent_data['message'][0]['eventForInvitee']['eventCode'];
    $e_name = $cvent_data['message'][0]['eventForInvitee']['eventTitle'];

    $token = access_token();

    $event_response_crm = event_search($e_code, $token);

    $first_name = $cvent_data['message'][0]['firstName'];
    $last_name = $cvent_data['message'][0]['lastName'];
    $contact_email = $cvent_data['message'][0]['email'];
    $inviteeStatus = $cvent_data['message'][0]['inviteeStatus'];

    $contact_data = array(
        'data' => [
            array(
                'First_Name' => $first_name,
                'Last_Name' => $last_name,
                'Email' => $contact_email
            )
        ]
    );

    $c_data = search_contact($token, $contact_email);

    if ($c_data == "") {
        $contact_info = create_contact($contact_data, $token);

        $contact_id = json_decode($contact_info)->data[0]->details->id;
    } else {
        $contact_id = $c_data->data[0]->id;
    }

    // Wait for contact_id to be available
    $maxRetries = 10;
    $retryCount = 0;

    while (empty($contact_id) && $retryCount < $maxRetries) {
        sleep(2); // Wait for 2 seconds before retrying
        $c_data = search_contact($token, $contact_email);
        if (!empty($c_data->data[0]->id)) {
            $contact_id = $c_data->data[0]->id;
        }
        $retryCount++;
    }

    if ($event_response_crm == "") {
        $e_id_crm = event_create($e_code, $e_name, $e_type, $token);
    } else {
        $e_id_crm = $event_response_crm->data[0]->id;
    }

    // Wait for e_id_crm to be available
    $retryCount = 0;

    while (empty($e_id_crm) && $retryCount < $maxRetries) {
        sleep(2); // Wait for 2 seconds before retrying
        $event_response_crm = event_search($e_code, $token);
        if (!empty($event_response_crm->data[0]->id)) {
            $e_id_crm = $event_response_crm->data[0]->id;
        }
        $retryCount++;
    }

    if (!empty($e_id_crm) && !empty($contact_id) && $e_type == 'InviteeOrGuestEmailSent' && $inviteeStatus!=="Declined" || $inviteeStatus!=="Accepted") {
        $inputs = [
            'data' => [
                [
                    'id' => $e_id_crm,
                    'Member_Status' => 'Active'
                ]
            ]
        ];

        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => "https://www.zohoapis.in/crm/v6/Contacts/$contact_id/Campaigns",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'PUT',
            CURLOPT_POSTFIELDS => json_encode($inputs),
            CURLOPT_HTTPHEADER => array(
                "Authorization: Zoho-oauthtoken $token",
                "Content-Type: application/json"
            ),
        ));

        $response = curl_exec($curl);
        curl_close($curl);
        echo $response;
    } else {
        echo json_encode(['message' => 'something wrong']);
    }
} catch (Exception $e) {
    // Log any exceptions
    logMessage("Exception: " . $e->getMessage());
}
?>
